/*
attribute highp vec4 a_vertex;
attribute mediump vec3 a_normal;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_ModelViewProj;
uniform lowp vec3 u_eyePosition;

uniform lowp vec3 u_lightAmbient;
uniform lowp vec3 u_lightDiffuse;
uniform lowp vec3 u_lightSpecular;
uniform highp vec3 u_lightPosition;

uniform lowp vec3 u_materialAmbient;
uniform lowp vec3 u_materialDiffuse;
uniform lowp vec3 u_materialSpecular;
uniform mediump float u_materialShininess;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;
varying lowp vec4 v_diffuse;

void main()
{
	gl_Position = u_ModelViewProj * a_vertex;
	
	lowp vec3 N = normalize( a_normal );
	
	//　Ambient Color
	lowp vec3 ambient = u_materialAmbient * u_lightAmbient;
	
	//　Diffuse Color
	lowp vec3 L = normalize(u_lightPosition - a_vertex.xyz);
	mediump float diffuseLight = max(dot(L, N), 0.0);
	mediump vec3 diffuse = u_materialDiffuse * u_lightDiffuse * diffuseLight;
	
	// Specular Color
	lowp vec3 V = normalize(u_eyePosition - a_vertex.xyz);
	lowp vec3 R = reflect( -V, N );	//　反射ベクトル -V + 2.0 * dot(N, V) * N;
	mediump float specularLight = pow(max(dot(L, R), 0.0), u_materialShininess);
	mediump vec3 specular = u_materialSpecular * u_lightSpecular * specularLight;

	v_color = a_color;
	v_diffuse = vec4(ambient + diffuse + specular, 1.0);
	v_texCoord = a_texCoord;
}
*/
attribute highp vec4 a_vertex;
attribute mediump vec3 a_normal;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_ModelViewProj;
uniform lowp vec3 u_eyePosition;

uniform lowp vec3 u_lightAmbient;
uniform lowp vec3 u_lightDiffuse;
uniform lowp vec3 u_lightSpecular;
uniform highp vec3 u_lightPosition;
uniform mediump float u_lightConstantAttenuation;
uniform mediump float u_lightLinearAttenuation;
uniform mediump float u_lightQuadraticAttenuation;

uniform lowp vec3 u_materialAmbient;
uniform lowp vec3 u_materialDiffuse;
uniform lowp vec3 u_materialSpecular;
uniform mediump float u_materialShininess;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;
varying lowp vec4 v_diffuse;

void main()
{
	gl_Position = u_ModelViewProj * a_vertex;
	
	lowp vec3 N = normalize( a_normal );
	
	//　Ambient Color
	lowp vec3 ambient = u_materialAmbient * u_lightAmbient;
	
	//　Diffuse Color
	// 光源ベクトル
	vec3 L = u_lightPosition - a_vertex.xyz;
	// 光源までの距離
	float dis = length(L);
	// ベクトルの正規化
	L = normalize(L);
	// 減衰係数
	float attenuation = 1.0 / (u_lightConstantAttenuation +
	                           u_lightLinearAttenuation * dis +
	                           u_lightQuadraticAttenuation * dis * dis);
//	float attenuation = 0.1;
							   
	mediump float diffuseLight = max(dot(L, N), 0.0);
	mediump vec3 diffuse = u_materialDiffuse * u_lightDiffuse * (diffuseLight * attenuation);
	
	// Specular Color
	lowp vec3 V = normalize(u_eyePosition - a_vertex.xyz);
	lowp vec3 R = reflect( -V, N );	//　反射ベクトル -V + 2.0 * dot(N, V) * N;
	mediump float specularLight = pow(max(dot(L, R), 0.0), u_materialShininess);
	mediump vec3 specular = u_materialSpecular * u_lightSpecular * (specularLight * attenuation);

	v_color = a_color;
	v_diffuse = vec4(ambient + diffuse + specular, 1.0);
	v_texCoord = a_texCoord;
}




