attribute highp vec4 a_vertex;
attribute mediump vec3 a_normal;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_ModelViewProj;
uniform lowp vec3 u_eyePosition;

uniform lowp vec3 u_lightAmbient;
uniform lowp vec3 u_lightDiffuse;
uniform lowp vec3 u_lightSpecular;
uniform highp vec3 u_lightPosition;

uniform lowp vec3 u_materialEmission;
uniform mediump float u_materialShininess;

uniform lowp float u_alpha;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;
varying lowp vec4 v_diffuse;

uniform highp mat4 u_ModelView;
//--------------------------------
// fog
//--------------------------------

uniform highp float u_fogParam1;
uniform highp float u_fogParam2;
uniform lowp vec4 u_fogColor;

varying lowp float v_fogFactor;
varying lowp vec4 v_fogColor;
//--------------------------------

void main()
{
	const vec3 Ambient = vec3( 0.5, 0.5, 0.5 );
	vec4 a_fogColor;
	
	vec3 N = normalize( a_normal );
	
	vec3 E = normalize( u_eyePosition - a_vertex.xyz );
	
	vec3 L = normalize( u_lightPosition - a_vertex.xyz );
	
	// ハーフベクトル
	vec3 H = normalize( L + E );
	
	//	vec3 Ambient = u_materialAmbient * u_lightAmbient;
	
	gl_Position = u_ModelViewProj * a_vertex;
	
	//	v_color = vec4( u_materialEmission + u_lightAmbient + Diffuse + Specular + Rim, u_alpha );
	//	v_color = vec4( u_materialEmission + Ambient +
	//				    u_materialDiffuse * u_lightDiffuse * max( dot( N, L ), 0.0 ) +
	//				    u_materialSpecular * u_lightSpecular * pow(max(dot(N, H), 0.0), u_materialShininess), u_alpha );
	v_color = a_color * vec4( u_materialEmission + u_lightAmbient +
	u_lightDiffuse * max( dot( N, L ), 0.0 ) +
	u_lightSpecular * pow(max(dot(N, H), 0.0), u_materialShininess), u_alpha );
	
	v_texCoord = a_texCoord;
	
	// 頂点を視点空間に座標変換する
	highp float z = (u_ModelView * a_vertex).z;
			
	// フォグ係数を[0,1]で作成
	v_fogFactor = clamp( u_fogParam1 + u_fogParam2 * z, 0.0, 1.0 );

	// フォグ係数とフォグの色から視点からの距離による色を作成
	a_fogColor = u_fogColor * ( 1.0 - v_fogFactor );
	v_fogColor = vec4( a_fogColor.xyz, 0.0 );
}
