attribute highp vec4 a_vertex;
attribute mediump vec3 a_normal;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_ModelViewProj;
uniform lowp vec3 u_eyePosition;

uniform lowp vec3 u_lightAmbient;
uniform lowp vec3 u_lightDiffuse;
uniform lowp vec3 u_lightSpecular;
uniform highp vec3 u_lightPosition;

uniform lowp vec3 u_materialAmbient;
uniform lowp vec3 u_materialDiffuse;
uniform lowp vec3 u_materialSpecular;
uniform mediump float u_materialShininess;

varying lowp vec4 v_color;
varying mediump vec2 v_texCoord;
varying lowp vec4 v_diffuse;

//--------------------------------
// fog
//--------------------------------
uniform highp mat4 u_ModelView;

uniform highp float u_fogParam1;
uniform highp float u_fogParam2;

varying lowp float v_fogFactor;
//--------------------------------

void main()
{
	gl_Position = u_ModelViewProj * a_vertex;
	
	lowp vec3 N = normalize( a_normal );
	
	//　Ambient Color
	lowp vec3 ambient = u_materialAmbient * u_lightAmbient;
	
	//　Diffuse Color
	lowp vec3 L = normalize(u_lightPosition - a_vertex.xyz);
	mediump float diffuseLight = max(dot(L, N), 0.0);
	mediump vec3 diffuse = u_materialDiffuse * u_lightDiffuse * diffuseLight;
	
	// Specular Color
	lowp vec3 V = normalize(u_eyePosition - a_vertex.xyz);
	lowp vec3 R = reflect( -V, N );	//　反射ベクトル -V + 2.0 * dot(N, V) * N;
	mediump float specularLight = pow(max(dot(L, R), 0.0), u_materialShininess);
	mediump vec3 specular = u_materialSpecular * u_lightSpecular * specularLight;

	v_color = a_color;
	v_diffuse = vec4(ambient + diffuse + specular, 1.0);
	v_texCoord = a_texCoord;
	
 	// 頂点を視点空間に座標変換する
	highp float z = (u_ModelView * a_vertex).z;
			
	// フォグ係数を[0,1]で作成
	v_fogFactor = clamp( u_fogParam1 + u_fogParam2 * z, 0.0, 1.0 );
}
