attribute highp vec4 a_vertex;
attribute mediump vec3 a_normal;
attribute lowp vec4 a_color;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_ModelViewProj;

uniform lowp vec3 u_lightAmbient;
uniform lowp vec3 u_lightDiffuse;
uniform highp vec3 u_lightPosition;

uniform lowp vec3 u_materialAmbient;
uniform lowp vec3 u_materialDiffuse;

varying lowp vec4 v_color;
varying lowp vec4 v_diffuse;
varying mediump vec2 v_texCoord;

//--------------------------------
// fog
//--------------------------------
uniform highp mat4 u_ModelView;

uniform highp float u_fogParam1;
uniform highp float u_fogParam2;
uniform lowp vec4 u_fogColor;

varying lowp float v_fogFactor;
varying lowp vec4 v_fogColor;
//--------------------------------

void main()
{
	gl_Position = u_ModelViewProj * a_vertex;
	
	lowp vec3 N = normalize( a_normal );
		
	//　Ambient Color
	lowp vec3 ambient = u_materialAmbient * u_lightAmbient;
	
	//　Diffuse Color
	lowp vec3 L = normalize(u_lightPosition - a_vertex.xyz);
	mediump float diffuseLight = max(dot(L, N), 0.0) * 0.5 + 0.5;
	mediump vec3 diffuse = u_materialDiffuse * u_lightDiffuse * (diffuseLight * diffuseLight);
	
	v_color = a_color;
	v_diffuse = vec4( ambient + diffuse, 1.0 );
	v_texCoord = a_texCoord;

	// 頂点を視点空間に座標変換する
	highp float z = (u_ModelView * a_vertex).z;
			
	// フォグ係数を[0,1]で作成
	v_fogFactor = clamp( u_fogParam1 + u_fogParam2 * z, 0.0, 1.0 );

	// フォグ係数とフォグの色から視点からの距離による色を作成
	v_fogColor = u_fogColor * ( 1.0 - v_fogFactor );
}
